package CVSMonitor::MetaData::File;

# Implements information storage for a single file in
# a CVS repository. The focus is on creating all processor
# intensive work at the creation time, since this will likely
# be done in the background once, and the time of construction
# is not important.

use strict;
use UNIVERSAL 'isa';
use Sort::Versions ();
use List::Util 'first';





#####################################################################
# Access methods

# Get our various parents
sub Module     { $_[0]->{_PARENT} }
sub Repository { $_[0]->{_PARENT}->Repository }
sub MetaData   { $_[0]->{_PARENT}->MetaData }

# The number of versions
sub versions { scalar @{ $_[0]->{_VERSIONS} } }

# Are we a binary file?
sub binary { $_[0]->{binary} }

# Get the size of the file on disk ( in lines ) in the working area.
# Won't work for binary files.
sub _getLinesOnDisk {
	my $self = shift;
	return undef if $self->binary;
	return undef unless defined $self->{working_file};

	# Fully resolve the file name
	my $file = File::Spec->catfile( $self->Module->pathCheckoutDir, $self->{working_file} );
	return undef unless $file;

	# Does the file exist?
	unless ( -e $file ) {
		### TEMPORARY FIX
		# For the rare case that somebody DOESN'T start their versions at 1.1,
		# we add a workaround by saying that the first version has 0 lines.
		# This is technically incorrect, and will lead to slightly corrupt
		# statistics, BUT means that the entire repository doesn't break.
		return 0;
	}

	# Open the file
	open ( LINESONDISK, $file ) or return undef;

	# Iterate through the file and count
	my $counter = 0;
	$counter++ while <LINESONDISK>;

	# Clean up and return the value
	close LINESONDISK;
	$counter;
}

# Get an array ref of all the versions
sub getVersions { $_[0]->{_VERSIONS} }

# Find a particular version, or 0 if not found
sub getVersion {
	my $self = shift;
	my $revision = shift or return undef;

	# Search through the Versions for it
	first { defined $_->{revision} 
		and $_->{revision} eq $revision } @{ $self->{_VERSIONS} }
		or 0;
}

# Finds the first version.
# The first version will always be the one with the lowest
# revision number. Because these are sorted lowest up on 
# creation. The 'creation' version will be the first one
# in the array.
sub getFirstVersion { 
	$_[0]->{_VERSIONS}->[0] 
	|| die "File '$_[0]->{working_file}' does not appear to have any versions"
}

# Get the most recent version.
# Because of the multiple branch thing, the most recent change
# might not be on the HEAD.
# So, find the one with the highest rawDate ( seconds past the epoch )
sub getRecentVersion {
	my $self = shift;

	# Start with what is sure to be the oldest
	my $Recent = $self->getFirstVersion;

	# Search through for the newest
	foreach my $Version ( @{ $self->getVersions } ) {
		if ( $Version->dateRaw > $Recent->dateRaw ) {
			$Recent = $Version;
		}
	}

	$Recent;
}

# Get the most recent HEAD version.
# This SHOULD be the version with only one . in it that is the highest
sub getRecentHeadVersion {
	my $self = shift;

	# Start with the oldest
	my $Recent = $self->getFirstVersion;

	foreach my $Version ( @{ $self->getVersions } ) {
		next unless $Version->revision =~ /^\d+\.\d+$/;
		if ( $Version->dateRaw > $Recent->dateRaw ) {
			$Recent = $Version;
		}
	}

	$Recent;
}

# Pull an annotated view of the file from the server
sub annotate { $_[0]->getRecentHeadVersion->annotate }

# Get the contents for the most recent HEAD version
sub contents { $_[0]->getRecentHeadVersion->contents }






#####################################################################
# Backend Methods
# The implementation of these methods are backend specific, and are
# passed through to the backend module.

# Is the file binary?
sub new      { shift; CVSMonitor::Backend->File_new( @_ ) }
sub filename { CVSMonitor::Backend->File_filename( shift ) }
sub diff     { CVSMonitor::Backend->File_diff( @_ ) }





#####################################################################
# Utility methods

sub _error { $CVSMonitor::MetaData::errstr = $_[1]; undef }
sub errstr   { $CVSMonitor::MetaData::errstr }

1;

__END__

# Copyright (C) 2002-2004 Adam Kennedy
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Should you wish to utilise this software under a different licence, 
# please contact the author.
